am 8d5aeffa: am 089d1615: am 01dbfbf9: am efd88e2c: Merge "Permit deactivation of malformed Device Admins." into jb-dev
* commit '8d5aeffa7dc27eba60fe4c64be6a89e645091958':
Permit deactivation of malformed Device Admins.
diff --git a/src/com/android/settings/DeviceAdminAdd.java b/src/com/android/settings/DeviceAdminAdd.java
index c0d615c..f668a63 100644
--- a/src/com/android/settings/DeviceAdminAdd.java
+++ b/src/com/android/settings/DeviceAdminAdd.java
@@ -116,34 +116,38 @@
return;
}
- // Make sure the given component name is actually a valid device admin.
- List<ResolveInfo> avail = getPackageManager().queryBroadcastReceivers(
- new Intent(DeviceAdminReceiver.ACTION_DEVICE_ADMIN_ENABLED),
- 0);
- int count = avail == null ? 0 : avail.size();
- boolean found = false;
- for (int i=0; i<count; i++) {
- ResolveInfo ri = avail.get(i);
- if (ai.packageName.equals(ri.activityInfo.packageName)
- && ai.name.equals(ri.activityInfo.name)) {
- try {
- // We didn't retrieve the meta data for all possible matches, so
- // need to use the activity info of this specific one that was retrieved.
- ri.activityInfo = ai;
- DeviceAdminInfo dpi = new DeviceAdminInfo(this, ri);
- found = true;
- } catch (XmlPullParserException e) {
- Log.w(TAG, "Bad " + ri.activityInfo, e);
- } catch (IOException e) {
- Log.w(TAG, "Bad " + ri.activityInfo, e);
+ // When activating, make sure the given component name is actually a valid device admin.
+ // No need to check this when deactivating, because it is safe to deactivate an active
+ // invalid device admin.
+ if (!mDPM.isAdminActive(cn)) {
+ List<ResolveInfo> avail = getPackageManager().queryBroadcastReceivers(
+ new Intent(DeviceAdminReceiver.ACTION_DEVICE_ADMIN_ENABLED),
+ 0);
+ int count = avail == null ? 0 : avail.size();
+ boolean found = false;
+ for (int i=0; i<count; i++) {
+ ResolveInfo ri = avail.get(i);
+ if (ai.packageName.equals(ri.activityInfo.packageName)
+ && ai.name.equals(ri.activityInfo.name)) {
+ try {
+ // We didn't retrieve the meta data for all possible matches, so
+ // need to use the activity info of this specific one that was retrieved.
+ ri.activityInfo = ai;
+ DeviceAdminInfo dpi = new DeviceAdminInfo(this, ri);
+ found = true;
+ } catch (XmlPullParserException e) {
+ Log.w(TAG, "Bad " + ri.activityInfo, e);
+ } catch (IOException e) {
+ Log.w(TAG, "Bad " + ri.activityInfo, e);
+ }
+ break;
}
- break;
}
- }
- if (!found) {
- Log.w(TAG, "Request to add invalid device admin: " + cn);
- finish();
- return;
+ if (!found) {
+ Log.w(TAG, "Request to add invalid device admin: " + cn);
+ finish();
+ return;
+ }
}
ResolveInfo ri = new ResolveInfo();